@@ -8,7 +8,7 @@ use Scalar::Util 'blessed';
 
 has 'error';
 
-our @EXPORT_OK = ('j');
+our @EXPORT_OK = qw(decode_json encode_json j);
 
 # Literal names
 my $FALSE = bless \(my $false = 0), 'Mojo::JSON::_Bool';
@@ -41,69 +41,63 @@ my $UTF_PATTERNS = {
 my $WHITESPACE_RE = qr/[\x20\x09\x0a\x0d]*/;
 
 sub decode {
-  my ($self, $bytes) = @_;
+  my $self = shift->error(undef);
 
-  # Clean start
-  $self->error(undef);
+  my $ref = eval { decode_json(shift) };
+  return $ref if $ref;
+
+  chomp(my $e = $@);
+  $self->error($e);
+  return undef;
+}
+
+sub decode_json {
 
   # Missing input
-  $self->error('Missing or empty input') and return undef unless $bytes;
+  die "Missing or empty input\n" unless length(my $bytes = shift);
 
   # Remove BOM
   $bytes =~ s/^(?:\357\273\277|\377\376\0\0|\0\0\376\377|\376\377|\377\376)//g;
 
   # Wide characters
-  $self->error('Wide character in input') and return undef
-    unless utf8::downgrade($bytes, 1);
+  die "Wide character in input\n" unless utf8::downgrade($bytes, 1);
 
   # Detect and decode Unicode
   my $encoding = 'UTF-8';
   $bytes =~ $UTF_PATTERNS->{$_} and $encoding = $_ for keys %$UTF_PATTERNS;
-  $bytes = Mojo::Util::decode $encoding, $bytes;
+  local $_ = Mojo::Util::decode $encoding, $bytes;
 
-  # Object or array
-  my $res = eval {
-    local $_ = $bytes;
+  # Leading whitespace
+  m/\G$WHITESPACE_RE/gc;
 
-    # Leading whitespace
-    m/\G$WHITESPACE_RE/gc;
+  # Array
+  my $ref;
+  if (m/\G\[/gc) { $ref = _decode_array() }
 
-    # Array
-    my $ref;
-    if (m/\G\[/gc) { $ref = _decode_array() }
+  # Object
+  elsif (m/\G\{/gc) { $ref = _decode_object() }
 
-    # Object
-    elsif (m/\G\{/gc) { $ref = _decode_object() }
+  # Invalid character
+  else { _exception('Expected array or object') }
 
-    # Invalid character
-    else { _exception('Expected array or object') }
-
-    # Leftover data
-    unless (m/\G$WHITESPACE_RE\z/gc) {
-      my $got = ref $ref eq 'ARRAY' ? 'array' : 'object';
-      _exception("Unexpected data after $got");
-    }
-
-    $ref;
-  };
-
-  # Exception
-  if (!$res && (my $e = $@)) {
-    chomp $e;
-    $self->error($e);
+  # Leftover data
+  unless (m/\G$WHITESPACE_RE\z/gc) {
+    my $got = ref $ref eq 'ARRAY' ? 'array' : 'object';
+    _exception("Unexpected data after $got");
   }
 
-  return $res;
+  return $ref;
 }
 
-sub encode { Mojo::Util::encode 'UTF-8', _encode_value($_[1]) }
+sub encode { encode_json($_[1]) }
+
+sub encode_json { Mojo::Util::encode 'UTF-8', _encode_value(shift) }
 
 sub false {$FALSE}
 
 sub j {
-  my $d = shift;
-  return __PACKAGE__->new->encode($d) if ref $d eq 'ARRAY' || ref $d eq 'HASH';
-  return __PACKAGE__->new->decode($d);
+  return encode_json($_[0]) if ref $_[0] eq 'ARRAY' || ref $_[0] eq 'HASH';
+  return eval { decode_json($_[0]) };
 }
 
 sub true {$TRUE}
@@ -305,7 +299,7 @@ sub _exception {
   my $context = 'Malformed JSON: ' . shift;
   if (m/\G\z/gc) { $context .= ' before end of data' }
   else {
-    my @lines = split /\n/, substr($_, 0, pos);
+    my @lines = split "\n", substr($_, 0, pos);
     $context .= ' at line ' . @lines . ', offset ' . length(pop @lines || '');
   }
 
@@ -381,6 +375,20 @@ C<u2028> and C<u2029> will always be escaped to make JSONP easier.
 L<Mojo::JSON> implements the following functions, which can be imported
 individually.
 
+=head2 decode_json
+
+  my $array = decode_json($bytes);
+  my $hash  = decode_json($bytes);
+
+Decode JSON to Perl data structure and die if decoding fails.
+
+=head2 encode_json
+
+  my $bytes = encode_json([1, 2, 3]);
+  my $bytes = encode_json({foo => 'bar'});
+
+Encode Perl data structure to JSON.
+
 =head2 j
 
   my $bytes = j([1, 2, 3]);
