@@ -157,9 +157,9 @@ sub render {
   my $maybe   = delete $args->{'mojo.maybe'};
 
   # Render
+  my $partial = $args->{partial};
   my ($output, $format) = $app->renderer->render($self, $args);
-  return defined $output ? Mojo::ByteStream->new($output) : undef
-    if $args->{partial};
+  return defined $output ? Mojo::ByteStream->new($output) : undef if $partial;
 
   # Maybe
   return $maybe ? undef : !$self->render_not_found unless defined $output;
@@ -172,55 +172,13 @@ sub render {
   return !!$self->rendered($self->stash->{status});
 }
 
-sub render_exception {
-  my ($self, $e) = @_;
-
-  my $app = $self->app;
-  $app->log->error($e = Mojo::Exception->new($e));
-
-  # Filtered stash snapshot
-  my $stash = $self->stash;
-  my %snapshot = map { $_ => $stash->{$_} }
-    grep { !/^mojo\./ and defined $stash->{$_} } keys %$stash;
-
-  # Render with fallbacks
-  my $mode     = $app->mode;
-  my $renderer = $app->renderer;
-  my $options  = {
-    exception => $e,
-    snapshot  => \%snapshot,
-    template  => "exception.$mode",
-    format    => $stash->{format} || $renderer->default_format,
-    handler   => undef,
-    status    => 500
-  };
-  my $inline = $renderer->_bundled(
-    $mode eq 'development' ? 'exception.development' : 'exception');
-  return $self if _fallbacks($self, $options, 'exception', $inline);
-  _fallbacks($self, {%$options, format => 'html'}, 'exception', $inline);
-  return $self;
-}
+sub render_exception { _development('exception', @_) }
 
 sub render_later { shift->stash('mojo.rendered' => 1) }
 
 sub render_maybe { shift->render(@_, 'mojo.maybe' => 1) }
 
-sub render_not_found {
-  my $self = shift;
-
-  # Render with fallbacks
-  my $app      = $self->app;
-  my $mode     = $app->mode;
-  my $renderer = $app->renderer;
-  my $format   = $self->stash->{format} || $renderer->default_format;
-  my $options
-    = {template => "not_found.$mode", format => $format, status => 404};
-  my $inline = $renderer->_bundled(
-    $mode eq 'development' ? 'not_found.development' : 'not_found');
-  return $self if _fallbacks($self, $options, 'not_found', $inline);
-  _fallbacks($self, {%$options, format => 'html'}, 'not_found', $inline);
-  return $self;
-}
+sub render_not_found { _development('not_found', @_) }
 
 sub render_static {
   my ($self, $file) = @_;
@@ -440,6 +398,34 @@ sub write_chunk {
   return $self->rendered;
 }
 
+sub _development {
+  my ($page, $self, $e) = @_;
+
+  my $app = $self->app;
+  $app->log->error($e = Mojo::Exception->new($e)) if $page eq 'exception';
+
+  # Filtered stash snapshot
+  my $stash = $self->stash;
+  my %snapshot = map { $_ => $stash->{$_} }
+    grep { !/^mojo\./ and defined $stash->{$_} } keys %$stash;
+
+  # Render with fallbacks
+  my $mode     = $app->mode;
+  my $renderer = $app->renderer;
+  my $options  = {
+    exception => $page eq 'exception' ? $e : undef,
+    format => $stash->{format} || $renderer->default_format,
+    handler  => undef,
+    snapshot => \%snapshot,
+    status   => $page eq 'exception' ? 500 : 404,
+    template => "$page.$mode"
+  };
+  my $inline = $renderer->_bundled($mode eq 'development' ? $mode : $page);
+  return $self if _fallbacks($self, $options, $page, $inline);
+  _fallbacks($self, {%$options, format => 'html'}, $page, $inline);
+  return $self;
+}
+
 sub _fallbacks {
   my ($self, $options, $template, $inline) = @_;
 
@@ -545,7 +531,7 @@ implements the following new ones.
 
   $c->continue;
 
-Continue dispatch chain.
+Continue dispatch chain with L<Mojolicious::Routes/"continue">.
 
 =head2 cookie
 
@@ -645,6 +631,9 @@ For more control you can also access request information directly.
   # Only GET parameters
   my $foo = $c->req->url->query->param('foo');
 
+  # Only POST parameters
+  my $foo = $c->req->body_params->param('foo');
+
   # Only GET and POST parameters
   my $foo = $c->req->param('foo');
 
@@ -660,25 +649,22 @@ For more control you can also access request information directly.
 
 Prepare a C<302> redirect response, takes the same arguments as L</"url_for">.
 
-  # Conditional redirect
-  return $c->redirect_to('login') unless $c->session('user');
-
   # Moved permanently
   $c->res->code(301);
   $c->redirect_to('some_route');
 
 =head2 render
 
-  my $bool    = $c->render;
-  my $bool    = $c->render(controller => 'foo', action => 'bar');
-  my $bool    = $c->render(template => 'foo/index');
-  my $bool    = $c->render(template => 'index', format => 'html');
-  my $bool    = $c->render(data => $bytes);
-  my $bool    = $c->render(text => 'Hello!');
-  my $bool    = $c->render(json => {foo => 'bar'});
-  my $bool    = $c->render(handler => 'something');
-  my $bool    = $c->render('foo/index');
-  my $output  = $c->render('foo/index', partial => 1);
+  my $bool   = $c->render;
+  my $bool   = $c->render(controller => 'foo', action => 'bar');
+  my $bool   = $c->render(template => 'foo/index');
+  my $bool   = $c->render(template => 'index', format => 'html');
+  my $bool   = $c->render(data => $bytes);
+  my $bool   = $c->render(text => 'Hello!');
+  my $bool   = $c->render(json => {foo => 'bar'});
+  my $bool   = $c->render(handler => 'something');
+  my $bool   = $c->render('foo/index');
+  my $output = $c->render('foo/index', partial => 1);
 
 Render content using L<Mojolicious::Renderer/"render"> and emit hooks
 L<Mojolicious/"before_render"> as well as L<Mojolicious/"after_render"> if the
@@ -687,6 +673,25 @@ controller and action or route name will be generated with
 L<Mojolicious::Renderer/"template_for">, all additional values get merged into
 the L</"stash">.
 
+  # Render characters
+  $c->render(text => 'I ♥ Mojolicious!');
+
+  # Render binary data
+  use Mojo::JSON 'j';
+  $c->render(data => j({test => 'I ♥ Mojolicious!'}));
+
+  # Render JSON
+  $c->render(json => {test => 'I ♥ Mojolicious!'});
+
+  # Render template "foo/bar.html.ep"
+  $c->render(template => 'foo/bar', format => 'html', handler => 'ep');
+
+  # Render template "foo/bar.*.*"
+  $c->render(template => 'foo/bar');
+
+  # Render template "test.xml.*"
+  $c->render('test', format => 'xml');
+
 =head2 render_exception
 
   $c = $c->render_exception('Oops!');
@@ -727,7 +732,9 @@ could be generated, takes the same arguments as L</"render">.
   $c = $c->render_not_found;
 
 Render the not found template C<not_found.$mode.$format.*> or
-C<not_found.$format.*> and set the response status code to C<404>.
+C<not_found.$format.*> and set the response status code to C<404>. Also sets
+the stash value C<snapshot> to a copy of the L</"stash"> for use in the
+templates.
 
 =head2 render_static
 
@@ -746,6 +753,14 @@ method does not protect from traversing to parent directories.
 Finalize response and emit hook L<Mojolicious/"after_dispatch">, defaults to
 using a C<200> response code.
 
+  # Custom response
+  $self->res->headers->content_type('text/plain');
+  $self->res->body('Hello World!');
+  $self->rendered(200);
+
+  # Accept WebSocket handshake without subscribing to an event
+  $self->rendered(101);
+
 =head2 req
 
   my $req = $c->req;
@@ -762,6 +777,7 @@ Get L<Mojo::Message::Request> object from L<Mojo::Transaction/"req">.
   my $agent    = $c->req->headers->user_agent;
   my $bytes    = $c->req->body;
   my $str      = $c->req->text;
+  my $hash     = $c->req->params->to_hash;
   my $hash     = $c->req->json;
   my $foo      = $c->req->json('/23/foo');
   my $dom      = $c->req->dom;
@@ -800,6 +816,9 @@ is set to the value C<XMLHttpRequest>.
     any  => {data => '', status => 204}
   );
 
+For more advanced negotiation logic you can also use the helper
+L<Mojolicious::Plugin::DefaultHelper/"accepts">.
+
 =head2 send
 
   $c = $c->send({binary => $bytes});
@@ -846,9 +865,10 @@ timeout, which usually defaults to C<15> seconds.
   $c          = $c->session({foo => 'bar'});
   $c          = $c->session(foo => 'bar');
 
-Persistent data storage, all session data gets serialized with L<Mojo::JSON>
-and stored C<Base64> encoded in C<HMAC-SHA1> signed cookies. Note that cookies
-usually have a 4096 byte limit, depending on browser.
+Persistent data storage for the next few requests, all session data gets
+serialized with L<Mojo::JSON> and stored C<Base64> encoded in C<HMAC-SHA1>
+signed cookies. Note that cookies usually have a 4096 byte limit, depending on
+browser.
 
   # Manipulate session
   $c->session->{foo} = 'bar';
@@ -882,12 +902,13 @@ discarded.
   $c       = $c->stash({foo => 'bar'});
   $c       = $c->stash(foo => 'bar');
 
-Non persistent data storage and exchange, application wide default values can
-be set with L<Mojolicious/"defaults">. Some stash values have a special
-meaning and are reserved, the full list is currently C<action>, C<app>, C<cb>,
-C<controller>, C<data>, C<extends>, C<format>, C<handler>, C<json>, C<layout>,
-C<namespace>, C<partial>, C<path>, C<status>, C<template> and C<text>. Note
-that all stash values with a C<mojo.*> prefix are reserved for internal use.
+Non-persistent data storage and exchange for the current request, application
+wide default values can be set with L<Mojolicious/"defaults">. Some stash
+values have a special meaning and are reserved, the full list is currently
+C<action>, C<app>, C<cb>, C<controller>, C<data>, C<extends>, C<format>,
+C<handler>, C<json>, C<layout>, C<namespace>, C<partial>, C<path>, C<status>,
+C<template> and C<text>. Note that all stash values with a C<mojo.*> prefix
+are reserved for internal use.
 
   # Remove value
   my $foo = delete $c->stash->{foo};
